@@ -1,66 +1,33 @@
 package Class::PObject::Driver::DBI;
 
-# $Id: DBI.pm,v 1.13 2003/11/07 04:51:04 sherzodr Exp $
+# $Id: DBI.pm,v 1.4 2003/08/23 14:31:29 sherzodr Exp $
 
 use strict;
-#use diagnostics;
 use Carp;
-use Log::Agent;
 use Class::PObject::Driver;
 use vars ('$VERSION', '@ISA');
 
 @ISA = ('Class::PObject::Driver');
-
-$VERSION = '2.02';
-
-
-
-sub _prepare_create_table {
-    my ($self, $object_name, $tablename) = @_;
-
-    my $props = $object_name->__props();
-    my @cols = ();
-    for my $column ( @{ $props->{columns} } ) {
-        my $type = $props->{tmap}->{$column};
-        if ( $type eq "MD5" ) {
-            $type = "CHAR(32)";
-        } elsif ( $type eq "ENCRYPT" ) {
-            $type = "CHAR(18)";
-        } else {
-            unless ( $type =~ m/^(CHAR|VARCHAR|INTEGER|TEXT|BLOB)(\([^\)]+\))?$/ ) {
-                logtrc 3, "%s is %s", $column, $type;
-                $type = "VARCHAR(255)"
-            }
-        }
-        if ( $column eq 'id' ) {
-            push @cols, "id INTEGER PRIMARY KEY NOT NULL";
-        } else {
-            push @cols, sprintf "%s %s NULL", $column, $type;
-        }
-    }
-    my $sql =  sprintf "\nCREATE TABLE %s (\n\t%s\n)", $tablename, join ",\n\t", @cols;
-    logtrc 4, $sql;
-    return $sql
-}
-
+$VERSION = '1.00';
 
 
 sub _prepare_where_clause {
     my ($self, $terms) = @_;
 
     $terms ||= {};
-    unless ( ref $terms ) {
-        die join ', ', caller(0)
-    }
+
     # if no terms present, just return an empty string
     unless ( keys %$terms ) {
         return ("", ());
     }
+
     my ($sql, @where, @bind_params);
+
     while ( my ($k, $v) = each %$terms ) {
         push @where, "$k=?";
         push @bind_params, $v
     }
+
     $sql = "WHERE " . join (" AND ", @where);
     return ($sql, \@bind_params)
 }
@@ -68,12 +35,13 @@ sub _prepare_where_clause {
 
 
 sub _prepare_select {
-    my ($self, $table_name, $terms, $args, $cols) = @_;
+    my ($self, $table_name, $terms, $args) = @_;
+
+    my ($sql, @where, @bind_params);
 
-    my ($sql, @where, @bind_params, $selected_cols);
     my ($where_clause, $bind_params) = $self->_prepare_where_clause($terms);
-    $selected_cols = $cols ? join (", ", @$cols) : "*";
-    $sql = sprintf("SELECT %s FROM %s %s", $selected_cols, $table_name, $where_clause);
+    $sql = "SELECT * FROM $table_name " . $where_clause;
+
     if ( defined $args ) {
         $args->{limit}      ||= 1000;
         $args->{offset}     ||= 0;
@@ -88,6 +56,7 @@ sub _prepare_select {
 
 
 
+
 sub _prepare_delete {
     my ($self, $table_name, $terms) = @_;
 
@@ -102,6 +71,7 @@ sub _prepare_delete {
 
 
 
+
 sub _prepare_insert {
     my ($self, $table_name, $columns) = @_;
 
@@ -120,6 +90,8 @@ sub _prepare_insert {
 
 
 
+
+
 sub _prepare_update {
     my ($self, $table_name, $columns, $terms) = @_;
 
@@ -141,6 +113,10 @@ sub _prepare_update {
 
 
 
+
+
+
+
 sub _tablename {
     my ($self, $object_name, $props, $dbh) = @_;
 
@@ -156,36 +132,24 @@ sub _tablename {
 
 
 
+
+
+
+
 sub load {
-    my $self = shift;
-    my ($object_name, $props, $id) = @_;
-
-    my $dbh = $self->dbh($object_name, $props) or return;
-    my $table = $self->_tablename($object_name, $props, $dbh) or return;
-    my ($sql, $bind_params) = $self->_prepare_select($table, {id=>$id});
-
-    my $sth = $dbh->prepare( $sql );
-    unless ( $sth->execute( @$bind_params ) ) {
-        $self->errstr($sth->errstr);
-        return undef
-    }
-
-    return $sth->fetchrow_hashref
-}
-
-
-
-sub load_ids {
     my $self = shift;
     my ($object_name, $props, $terms, $args) = @_;
 
-    my $dbh   = $self->dbh($object_name, $props)              or return;
+    if ( $terms && (ref($terms) ne 'HASH') && ($terms =~m/^\d+$/) ) {
+        $terms = {id => $_[2]}
+    }
+
+    my $dbh   = $self->dbh($object_name, $props)                  or return;
     my $table = $self->_tablename($object_name, $props, $dbh) or return;
-    my ($sql, $bind_params)   = $self->_prepare_select($table, $terms, $args, ['id']);
-
+    my ($sql, $bind_params)   = $self->_prepare_select($table, $terms, $args);
 
+    $self->_read_lock($dbh, $table) or return undef;
     my $sth   = $dbh->prepare( $sql );
-
     unless( $sth->execute(@$bind_params) ) {
         $self->errstr($sth->errstr);
         return undef
@@ -195,15 +159,20 @@ sub load_ids {
         return []
     }
 
-    my @data_set = ();
+    my @rows = ();
     while ( my $row = $sth->fetchrow_hashref() ) {
-        push @data_set, $row->{id}
+        push @rows, $row
     }
-    return \@data_set
+    $self->_unlock($dbh, $table);
+    return \@rows
 }
 
 
 
+
+
+
+
 sub remove {
     my $self = shift;
     my ($object_name, $props, $id)  = @_;
@@ -215,18 +184,23 @@ sub remove {
 
     my $dbh                 = $self->dbh($object_name, $props)              or return;
     my $table               = $self->_tablename($object_name, $props, $dbh) or return;
-    my ($sql, $bind_params) = $self->_prepare_delete($table, {id=>$id});
+    my ($sql, $bind_params) = $self->_prepare_delete($table);
+
+    $self->_write_lock($dbh, $table) or return undef;
 
     my $sth                 = $dbh->prepare( $sql );
     unless ( $sth->execute($id) ) {
         $self->errstr($sth->errstr);
         return undef
     }
+    $self->_unlock($dbh, $table) or return undef;
     return $id
 }
 
 
 
+
+
 sub remove_all {
     my $self  = shift;
     my ($object_name, $props, $terms) = @_;
@@ -235,31 +209,18 @@ sub remove_all {
     my $table               = $self->_tablename($object_name, $props, $dbh) or return;
     my ($sql, $bind_params) = $self->_prepare_delete($table, $terms);
 
+    $self->_write_lock($dbh, $table) or return undef;
     my $sth   = $dbh->prepare( $sql );
     unless ( $sth->execute(@$bind_params) ) {
         $self->errstr($sth->errstr);
         return undef
     }
+    $self->_unlock($dbh, $table) or return undef;
     return 1
 }
 
 
 
-sub drop_datasource {
-    my $self = shift;
-    my ($object_name, $props) = @_;
-
-    my $dbh = $self->dbh($object_name, $props) or return;
-    my $table= $self->_tablename($object_name, $props, $dbh) or return;
-    unless ( $dbh->do( "DROP TABLE $table" ) ) {
-        $self->errstr( $dbh->errstr );
-        return undef
-    }
-
-    return 1
-}
-
-
 
 sub count {
     my $self = shift;
@@ -270,6 +231,7 @@ sub count {
     my ($where_clause, $bind_params)= $self->_prepare_where_clause($terms);
     my $sql                         = "SELECT COUNT(*) FROM $table " . $where_clause;
 
+    $self->_read_lock($dbh, $table) or return undef;
     my $sth                         = $dbh->prepare( $sql );
     unless ( $sth->execute( @$bind_params ) ) {
         $self->errstr($sth->errstr);
@@ -277,11 +239,15 @@ sub count {
     }
 
     my $count = $sth->fetchrow_array || 0;
+    $self->_unlock($dbh, $table) or return undef;
     return $count
 }
 
 
 
+
+
+
 sub _read_lock {
     my ($self, $dbh, $table) = @_;
 
@@ -294,6 +260,10 @@ sub _read_lock {
 
 
 
+
+
+
+
 sub _write_lock {
     my ($self, $dbh, $table) = @_;
 
@@ -306,6 +276,8 @@ sub _write_lock {
 
 
 
+
+
 sub _unlock {
     my ($self, $dbh, $table) = @_;
 
@@ -324,6 +296,7 @@ sub _unlock {
 
 1;
 __END__
+# Below is stub documentation for your module. You'd better edit it!
 
 =head1 NAME
 
@@ -337,11 +310,13 @@ Class::PObject::Driver::DBI - Base class for all DBI-related drivers
 
     sub save {
         my ($self, $pobject_name, \%properties, \%columns) = @_;
+
         ...
     }
 
     sub dbh {
         my ($self, $pobject_name, \%properties) = @_;
+
         ...
     }
 
@@ -353,12 +328,12 @@ Class::PObject::Driver::DBI - Base class for all DBI-related drivers
 
 =head1 STOP!
 
-If you just want to be able to use L<Class::PObject|Class::PObject> this manual is not for you.
+If you just want to be able to use Class::PObject> this manual is not for you.
 This is for those willing to write I<pobject> drivers to support other database
 systems and storage devices.
 
 If you just want to be able to use Class::PObject, you should refer to its
-L<manual|Class::PObject> instead.
+L<online manual|Class::PObject> instead.
 
 =head1 DESCRIPTION
 
@@ -367,7 +342,7 @@ and overrides the methods provided in Class::PObject::Driver with those more rel
 to RDBMS engines.
 
 It uses ANSI-SQL syntax, so most of the base methods should perform as expected for most
-RDBMS that support ANSI-SQL syntax.
+RDBMS that supports ANSI-SQL syntax.
 
 For those that don't, you can override necessary methods from within your driver class.
 This manual will not discuss the list of base methods, for they all are documented in
@@ -388,8 +363,7 @@ For details on C<save()> method, refer to L<Class::PObject::Driver|Class::PObjec
 =item *
 
 C<dbh($self, $pobject_name, \%properties)> - will be called by other base methods whenever
-a database handle is needed. It receives all the standard arguments
-(see L<Class::PObject::Driver|Class::PObject::Driver>)
+a database handle is needed. It receives all the standard arguments (L<Class::PObject::Driver>)
 
 If your project consists of several pobjects, which is very common, you may want to C<stash()>
 the created database handle to ensure Class::PObject will be able to re-use the same object
@@ -429,8 +403,9 @@ Example:
 
 =item *
 
-C<_prepare_select($self, $table_name, \%terms, \%args, \@cols)> - prepares a I<SELECT> SQL statement,
-given $table_name, \%terms and \%args and \@cols.
+C<_prepare_select($self, $table_name, \%terms, \%args)> - prepares a I<SELECT> SQL statement,
+given $table_name, \%terms and \%args. The last two arguments are the same as the ones passed
+to C<load()> pobject method.
 
 Example:
 
@@ -463,16 +438,6 @@ C<_prepare_insert($self, $table_name, \%columns)> builds an I<INSERT> SQL statem
 
 C<_prepare_delete($self, $table_name, \%terms)> builds a I<DELETE> SQL statement
 
-
-=item *
-
-C<_prepare_create_table($self, $object_name, $table_name)> builds a I<CREATE TABLE> SQL
-statement. This can be used from within pobject drivers for creating a table if it's
-detected to be missing.
-
-Unlike other C<_prepare_*> methods, this method returns only one value on success, C<$sql>,
-which is a string containing SQL statement.
-
 =item *
 
 C<_tablename($self, $pobject_name, $props, $dbh)> returns a name of the table
@@ -493,8 +458,15 @@ it into SQL statements.
 
 L<Class::PObject::Driver>
 
+=head1 AUTHOR
+
+Sherzod B. Ruzmetov, E<lt>sherzodr@cpan.orgE<gt>, http://author.handalak.com/
+
 =head1 COPYRIGHT AND LICENSE
 
-For author and copyright information refer to Class::PObject's L<online manual|Class::PObject>.
+Copyright 2003 by Sherzod B. Ruzmetov.
+
+This library is free software; you can redistribute it and/or modify
+it under the same terms as Perl itself.
 
 =cut
